/*
 * ExperimentC.cc - Simple experiment manager.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 17-Jun-1998
 *
 */

#include "ExperimentC.h"



ExperimentC::ExperimentC
(
	ConfigurationA const & iConfiguration,
	StatisticsA & iStatistics,
	CoreFactoryA const & iCoreFactory,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	RandomA & iRandom
): ExperimentA(iConfiguration, iStatistics, iCoreFactory, iProblemFactory, iReproductionFactory, iRandom)
{
	set_new_handler(OutOfMemory);

	Random.SetSeed(Configuration("Experiment", "RandomSeed"));
	Problem = ProblemFactory.CreateProblem();


} // ExperimentC



ExperimentC::~ExperimentC()
{
	delete Problem;

} // ~ExperimentC



void
ExperimentC::Run()
{
	ProblemC * MyProblem = dynamic_cast<ProblemC*>(Problem);
	MyProblem->SetFilename(Configuration("Problem", "Filename"));
	Problem->Produce();

	Statistics.ResetRunStatistics();

	unsigned int NumberOfRuns = Configuration("Experiment", "NumberOfRuns");
	for (unsigned int i = 0; i < NumberOfRuns; ++i)
	{
		EvolutionaryAlgorithm = CoreFactory.CreateEvolutionaryAlgorithm(Statistics, ProblemFactory, ReproductionFactory, *Problem);

		EvolutionaryAlgorithm->Run();
		
		delete EvolutionaryAlgorithm;

		cout << Statistics << endl;
	}


} // Run



// Eof ExperimentC.cc
