/*
 * RowC.h - A base class for the problem class.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 05-Aug-1998 
 *
 */

#ifndef RowC_h
#define RowC_h 1

#include <typedef.h>

#include <vector>

class DataA;

template <class T>
class RowC
{
	public:

		RowC(){};
		virtual ~RowC (){};

		virtual void AddInput
		(
			T & iInput
		)
		{
			Inputs.push_back(iInput);
		};
			
		virtual T const & GetInput
		(
			unsigned int iInt
		)
		{
			return Inputs[iInt];
		};
			
		virtual void AddOutput
		(
			T & iOutput
		)
		{
			Outputs.push_back(iOutput);
		};
		
		friend ostream const & operator<<
		(
			ostream & iOstream,
			RowC<T> const * const iRow
		)
		{
			copy(iRow->Inputs.begin(), iRow->Inputs.end(), ostream_iterator<T>(iOstream, " "));

			if (!iRow->Outputs.empty())
				cout << ": ";

			copy(iRow->Outputs.begin(), iRow->Outputs.end(), ostream_iterator<T>(iOstream, " "));

			return iOstream;
		};

		friend ostream const & operator<<
		(
			ostream & iOstream,
			RowC<T> const & iRow
		)
		{
			copy(iRow.Inputs.begin(), iRow.Inputs.end(), ostream_iterator<T>(iOstream, " "));

			if (!iRow.Outputs.empty())
				cout << " : ";

			copy(iRow.Outputs.begin(), iRow.Outputs.end(), ostream_iterator<T>(iOstream, " "));

			return iOstream;
		};

	protected:

		vector<T> Inputs;

		vector<T> Outputs;

	private:

}; // RowC

#endif

// Eof RowC.h
