/*
 * TreeInitializationC.cc - Simple implementation of an initialization unit.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Aug-1998
 *
 */

#include "TreeInitializationC.h"
#include <Common/TreeC.tpl>


TreeInitializationC::TreeInitializationC
(
	ConfigurationA const & iConfiguration,
	ProblemA const & iProblem,
	RandomA const & iRandom
): InitializationA(iConfiguration, iProblem, iRandom)
{

} // TreeInitializationC



TreeInitializationC::~TreeInitializationC()
{
} // ~TreeInitializationC


void
TreeInitializationC::CreateFullTree
(
	TreeC<GP_NodeA<bool> >::Insert_IteratorC Root,
	unsigned int Depth
)
{
	if (Depth < Configuration("Genotype", "MaxTreeDepth"))
	{
		unsigned int r = Random(3);
		FunctionA<bool> * Node;
		switch (r)
		{
			case 0:
				{
					Node = new BooleanFunction_ONE_NOT_C;
					break;
				}
			case 1:
				{
					Node = new BooleanFunction_TWO_AND_C;
					break;
				}
			case 2:
				{
					Node = new BooleanFunction_TWO_OR_C;
					break;
				}
			default:
				throw new InternalErrorC("Tried to create a non-existing GP function, while initializing a tree.");
		}

		Root.AddChild(*Node);
		Depth++;

		CreateFullTree(Root, Depth);
		for (unsigned int i = 1; i < Node->Arity(); ++i)
		{
			CreateFullTree(Root, Depth);
		}

	}
	else
	{
		 
		unsigned int r = Random(1);
		TerminalA<bool> * Node;
		switch (r)
		{
			case 0:
				{
					Node = new VariableC<bool>(Random(Configuration("Problem","SizeOfRecord")));
					break;
				}
			case 1:
				{
					if (Random(2) == 0)
						Node = new ConstantC<bool>(false);
					else
						Node = new ConstantC<bool>(true);
					break;
				}
			default:
				throw new InternalErrorC("Tried to create a non-existing GP terminal, while initializing a tree.");
		}

		Root.AddChild(*Node);
	}
 
}

void
TreeInitializationC::operator()
(
	DataA & iData
) const
{
	DataA const * tmp = &iData;
	TreeDataC * TreeData = dynamic_cast<TreeDataC const *>(tmp);
	if (TreeData == 0)
	{
		 throw new InternalErrorC("Got an incompatible data structure in TreeInitializationC::operator().");
	}

	ProblemA const * tmp2 = &Problem;
	DataBaseProblemC const * DataBaseProblem = dynamic_cast<DataBaseProblemC const *>(tmp2);
	if (TreeData == 0)
	{
		 throw new InternalErrorC("Got an incompatible problem in TreeInitializationC::operator().");
	}

	TreeC<GP_NodeA<bool> >::Iterator_PostorderC i = TreeData->Tree.CreateIterator_Postorder();
	TreeC<GP_NodeA<bool> >::Insert_IteratorC ii = TreeData->Tree.CreateInsert_Iterator(i);

	CreateFullTree(ii, 0);

	//TODO: Remove debug stuff from TreeInitializationC::operator().
	cerr << "\n";
	cerr << TreeData->Tree << endl;

}



// Eof TreeInitializationC.cc
