
#include "DataBaseProblemC.h"

#include <ExperimentC.h>
#include <ConfigurationC.h>
#include <StatisticsA.h>
#include <Common/RandomC.h>

#include "MyCoreFactoryC.h"
#include "MyProblemFactoryC.h"
#include <Reproduction/ReproductionFactoryC.h>

#include <Error/ErrorC.h>
#include <Error/WarningC.h>
#include <Error/InternalErrorC.h>
#include <Error/LibraryErrorC.h>

int main()
{

	try
	{
		/*
		 *  By using a configuration file, we can control all kind of parameters
		 *  when the program is compiled.
		 */

		cerr << "Opening configuration file\n  [configuration]";

		ConfigurationA * configuration;
		configuration = new ConfigurationC("example.ini");

		/*
		 * We need a statistics class for collecting information.
		 *
		 */

		cerr << "\nCreating statistics\n  [statistics]";

		StatisticsA * statistics;
		statistics = new StatisticsC(*configuration);


		/*
		 *  The library needs a randomgenerator, which we will have to provide,
		 *  luckily the library also provides a simple generator.
		 */

		cerr << "\nCreating randomgenerator\n  [randomgenerator]";

		RandomA * randomgenerator;
		randomgenerator = new RandomC();


		/*
		 *  First things first, one factory has to be made for each
		 *  package.
		 */

		cerr << "\nCreating factories";

		cerr << "\n  [corefactory]";
		CoreFactoryA * corefactory;
		corefactory = new MyCoreFactoryC(*configuration, *randomgenerator);

		cerr << "\n  [problemfactory]";
		ProblemFactoryA * problemfactory;
		problemfactory = new MyProblemFactoryC(*configuration);

		cerr << "\n  [reproductionfactory]";
		ReproductionFactoryA * reproductionfactory;
		reproductionfactory = new ReproductionFactoryC(*configuration, *randomgenerator);

		/*
		 *  We let the Experiment class produce the problem and evolutionary
		 *  algorithm for us.
		 */

		cerr << "\nCreating experiment manager\n  [experiment]";

		ExperimentC * experiment;
		experiment = new ExperimentC(*configuration, *statistics, *corefactory, *problemfactory, *reproductionfactory, *randomgenerator);


		/*
		 *  All setup and ready to go, it is now time to run the experiment.
		 */

		cerr << "\nRunning experiment\n";

		experiment->Run ();


		/*
		 *  EA is done, time to shut down the program, but first we have
		 *  to clean up our memory.
		 */

		cerr << "\nFreeing memory";

		cerr << "\n  [experiment]";
		delete experiment;
		cerr << "\n  [problemfactory]";
		delete problemfactory;
		cerr << "\n  [corefactory]";
		delete corefactory;
		cerr << "\n  [reproductionfactory]";
		delete reproductionfactory;
		cerr << "\n  [randomgenerator]";
		delete randomgenerator;
		cerr << "\n  [statistics]";
		delete statistics;
		cerr << "\n  [configuration]";
		delete configuration;
	}
	catch (ExceptionA* e)
	{
		e->Handle();
		delete e;
	}
	catch (...)
	{
		cerr << "Signal 11. Someone has thrown an exception, but it is not clear who and of what kind it is.\n";
	}

	cerr << "\n";
	return 0;
}
