/*
 * GenotypeC.h - Abstract base class defining the interface of a genotype.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 03-Jun-1998
 *
 */

#ifndef GenotypeC_h
#define GenotypeC_h 1

#include "Core/GenotypeA.h"

#include "Core/CoreFactoryA.h"
//REM: #include "Problem/FitnessValueA.h"
#include "Problem/DataC.h"
#include "Problem/ProblemFactoryA.h"


class GenotypeC: public GenotypeA
{
	public:

		GenotypeC
		(
			ConfigurationA const & iConfiguration,
			ProblemFactoryA const & iProblemFactory
		);

		virtual ~GenotypeC();

		GenotypeC
		(
			ConfigurationA const & iConfiguration,
			ProblemFactoryA const & iProblemFactory,
			GenotypeA const & iGenotype
		);

		virtual void Init
		(
			InitializationA const & iInitialization
		);

		virtual void Evaluate
		(
			FitnessFunctionA const & iFitnessFunction
		);
		
		virtual GenotypeC const & operator=
		(
			GenotypeA const & iGenotype
		);

		virtual DataA & GetData() const
		{
			return *Data;
		};

		//REM: virtual void  SetData(DataA & iData);

		virtual FitnessType const & GetFitness() const
		{
			return FitnessValue;
		};

		//RM: virtual void  SetFitnessValue(FitnessValueA & iFitnessValue);

	protected:

		virtual void PrintToStream
		(
			ostream & in
		) const;

		/*
		virtual bool IsEqual
		(
			GenotypeA const & iGenotype
		) const;

		virtual bool IsSmaller
		(
			GenotypeA const & iGenotype
		) const;

		virtual bool IsGreater
		(
			GenotypeA const & iGenotype
		) const;
		*/

		virtual void  Copy
		(
			GenotypeA const & iGenotype
	 	);

		DataA * Data;

	private:

}; // GenotypeC

#endif

// Eof GenotypeC.h
