/*
 * InitializationA.h - Abstract base class defining the interface of an initialization unit.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 26-Jun-1998
 *
 */

#ifndef InitializationA_h
#define InitializationA_h 1

class RandomA;
class DataA;
class ConfigurationA;
class ProblemA;


/** With this class a genotype can initialize its data part. This will probably happen by a random fill of the data such that it does not violate the domain of the problem.
 *
 * @memo Initializing genotypes
 *
 * @author J.I. van Hemert
 * @version 0.1
 *
 */

class InitializationA
{
	public:

		InitializationA
		(
			ConfigurationA const & iConfiguration,
			ProblemA const & iProblem,
			RandomA const & iRandom
		);

		virtual ~InitializationA() =0;

		virtual void operator()
		(
			DataA & iData
		) const =0;

	protected:

		ConfigurationA const & Configuration;
		ProblemA const & Problem;
		RandomA const & Random;

	private:

}; // InitializationA

#endif

// Eof InitializationA.h
