/*
 * PoolReverseIteratorC.h - Simple reverse iterator to work with the PoolC class which implements a population as a simple list.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Jun-1998
 *
 */

#ifndef PoolReverseIteratorC_h
#define PoolReverseIteratorC_h 1

#include "Core/PoolIteratorA.h"

#include "Core/PoolA.h"
#include "Error/InternalErrorC.h"

class PoolReverseIteratorC: public PoolIteratorA
{
	public:
	
		PoolReverseIteratorC
		(
			PoolA & iPool
		);

		virtual ~PoolReverseIteratorC();

		virtual void First();

		virtual void Next();

		virtual void Delete();

		virtual GenotypeA & CurrentItem() const;

		virtual bool IsDone() const;

	protected:

		PoolElementA * Cursor;

		PoolA * Pool;

	private:

}; // PoolReverseIteratorC



class PoolReverseConstIteratorC: public PoolConstIteratorA
{
	public:
	
		PoolReverseConstIteratorC
		(
			PoolA const & iPool
		);

		virtual ~PoolReverseConstIteratorC();

		virtual void First();

		virtual void Next();

		virtual GenotypeA const & CurrentItem() const;

		virtual bool IsDone() const;

	protected:

		PoolElementA * Cursor;

		PoolA const * Pool;

	private:

}; // PoolReverseConstIteratorC

#endif

// Eof PoolReverseIteratorC.h
