/*
 * ConfigurationC.cc - A general configuration file reader with classes.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 24-May-1998
 *
 */

#include "ConfigurationC.h"
#include "Common/ArrayC.tpl"



ConfigurationC::ConfigurationC
(
	StringC const & iFileName
) throw(ExceptionA*)
{
	char EqualSine;
	ifstream FileData;
	StringC s;
	StringC Class;
	NumberOfLabels = 0;
	LargestString = 0;

	FileData.open(iFileName, ios::in);
	if (!FileData.is_open())
	{
		StringC ErrorString = "The configfile \"";
		ErrorString += iFileName;
		ErrorString += "\" could not be opened.";
		throw new ErrorC(ErrorString);
	}

	FileName = iFileName;

	while (!FileData.eof())
	{
		FileData >> s;
		if (LargestString < strlen(s))
			LargestString = strlen(s);
		if (strchr(s, '=') != 0)
			NumberOfLabels++;
	}
	Items = new ArrayC<ItemsT> (NumberOfLabels, "Configuration");

	FileData.close();
	FileData.open(iFileName, ios::in);

	unsigned int i = 0;
	while (!FileData.eof() && (i <= NumberOfLabels))
	{
		FileData >> s;

		if ((s [0] == '[' ) && (s [strlen(s) - 1] == ']'))
		{
			Class = s;
		}
		else
			if (s [0] != '\0')
			{
				(*Items) [i].Label = Class;
				(*Items) [i].Label += ".";
				(*Items) [i].Label += s;
				FileData >> EqualSine;
				if (EqualSine != '=')
				{
					StringC ErrorString = "Reading in \"";
					ErrorString	+= iFileName;
					ErrorString += "\" while looking at \"";
					ErrorString += s;
					ErrorString += "\" in class \"";
					ErrorString += Class;
					ErrorString += "\".";
					throw new ErrorC(ErrorString);
				}

				FileData >> s;
				(*Items) [i].Data = s;
				i++;
			}
	}

	if (!FileData.eof())
	{
		FileData.close();
		StringC ErrorString = "More labels read in then should be in file \"";
		ErrorString += iFileName;
		ErrorString += "\", please check the file for errors.";
		throw new ErrorC(ErrorString);
	}

	FileData.close();

} // ConfigurationC



ConfigurationC::~ConfigurationC()
{
	delete Items;

} // ~ConfigurationC



StringC const ConfigurationC::GetStringItem
(
	StringC const & iClass, 
	StringC const & iItem
) const throw(ExceptionA*)
{
	unsigned int i;
	StringC Search;

	Search = "[";
	Search += iClass;
	Search += "].";
	Search += iItem;

	i = 0;
	while ((i < NumberOfLabels) && (Search != ((*Items) [i].Label)))
	{
		i++;
	}

	if (i < NumberOfLabels)
	{
		return ((*Items) [i]).Data;
	}
	else
	{
		StringC ErrorString = "Label \"";
		ErrorString += StringC(iItem);
		ErrorString += "\" not found in \"";
		ErrorString += FileName;
		ErrorString += "\".";
		throw new WarningC(ErrorString);
		return ("");
	}

} // GetStringItem



StringC const ConfigurationC::operator()
(
	char const * iClass,
	char const * iItem
) const
{
	StringC Class(iClass);
	StringC Item(iItem);

	return GetStringItem(Class, Item);

} // operator()



StringC const ConfigurationC::operator()
(
	StringC const & iClass,
	char const * iItem
) const
{
	StringC Item(iItem);

	return GetStringItem(iClass, Item);

} // operator()



StringC const ConfigurationC::operator()
(
	char const * iClass,
	StringC const & iItem
) const
{
	StringC Class(iClass);

	return GetStringItem(Class, iItem);

} // operator()



StringC const ConfigurationC::operator()
(
	StringC const & iClass,
	StringC const & iItem
) const
{
	return GetStringItem(iClass, iItem);

} // operator()



void const
ConfigurationC::PrintToStream
(
	ostream & iStream
) const
{
	for (unsigned int i = 0; i < NumberOfLabels; i++)
		iStream << setw(iStream.width()) << (*Items) [i].Label << " = " << (*Items) [i].Data << endl;

} // PrintToStream



// Eof ConfigurationC.cc
