/*
 * CoreFactoryC.cc - Factory defaulting to the classes in the implementation level.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 26-Jun-1998
 *
 */

#include "Core/CoreFactoryC.h"


EvolutionaryAlgorithmA* 
CoreFactoryC::CreateEvolutionaryAlgorithm
(
	StatisticsA & iStatistics,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	ProblemA const & iProblem
) const
{
	return new EvolutionaryAlgorithmC(Configuration, iStatistics, *this, iProblemFactory, iReproductionFactory, iProblem);

} // CreateEvolutionaryAlgorithm



PopulationA* 
CoreFactoryC::CreatePopulation
(
	StatisticsA & iStatistics,
	ProblemFactoryA const & iProblemFactory,
	ReproductionFactoryA const & iReproductionFactory,
	StopConditionA const & iStopCondition,
	ProblemA const & iProblem
) const
{
	return new PopulationC(Configuration, iStatistics, *this, iProblemFactory, iReproductionFactory, iStopCondition, iProblem);

} // CreatePopulation



GenotypeA* 
CoreFactoryC::CreateGenotype
(
	ProblemFactoryA const & iProblemFactory
) const
{
	return new GenotypeC(Configuration, iProblemFactory);

} // CreateGenotype



StopConditionA* 
CoreFactoryC::CreateStopCondition() const
{
	return new StopConditionC();

} // CreateStopCondition



InitializationA* 
CoreFactoryC::CreateInitialization
(
	ProblemA const & iProblem
) const
{
	return new InitializationC(Configuration, iProblem, Random);

} // CreateInitialization



PoolA* 
CoreFactoryC::CreatePool() const
{
	return new PoolC();

} // CreatePool



// Eof CoreFactoryA.cc
