/*
 * PoolC.cc - Simple implementation of a population pool.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 11-Jun-1998
 *
 */

#include "Core/PoolC.h"



// Implementation PoolElement ---------------



PoolElementC::PoolElementC
(
	GenotypeA & iGenotype,
	PoolElementA & iPreviousPoolElement,
	PoolElementA & iNextPoolElement
): PoolElementA(iGenotype)
{
	NextPoolElement = &iNextPoolElement;
	PreviousPoolElement = &iPreviousPoolElement;

	if (NextPoolElement != 0)
		NextPoolElement->SetPreviousPoolElement(*this);

	if (PreviousPoolElement != 0)
		PreviousPoolElement->SetNextPoolElement(*this);
} // PoolElementC



PoolElementC::~PoolElementC()
{
	if (NextPoolElement != 0)
		NextPoolElement->SetPreviousPoolElement(*PreviousPoolElement);

	if (PreviousPoolElement != 0)
		PreviousPoolElement->SetNextPoolElement(*NextPoolElement);

} // ~PoolElementC



void
PoolElementC::SetPreviousPoolElement
(
	PoolElementA & iPoolElement
)
{
	PreviousPoolElement = &iPoolElement;

} // SetPreviousPoolElement



void
PoolElementC::SetNextPoolElement
(
	PoolElementA & iPoolElement
)
{
	NextPoolElement = &iPoolElement;

} // SetNextPoolElement



// Implementation Pool ---------------

PoolC::PoolC()
{
	ThePool = 0;
	PoolEnd = 0;
} // PoolC



//TODO: Finish copy ctors, and add a method for building iterators.
/*PoolC::PoolC
(
	PoolC const & iPool
)
{
} // PoolC


const & PoolC
PoolC::operator=
(
	PoolC const & iPool
)
{
	
} // operator=
*/



PoolC::~PoolC()
{
	if (Size > 0)
	{
		PoolIteratorC PoolIterator(*this);

		PoolIterator.First();
		while (!PoolIterator.IsDone())
		{
			delete &PoolIterator.CurrentItem();
			PoolIterator.Delete();
		}
	}

} // ~PoolC



/*
void
PoolC::Add
(
	GenotypeA & iGenotype
)
{
	PoolElementC * Temp;
	
	Temp = new PoolElementC(iGenotype,*List);

	List = Temp;

	++Size;

} // Add
*/


void
PoolC::Delete
(
	PoolElementA & iPoolElement
)
{
	if (&iPoolElement == ThePool)
		ThePool = &(ThePool->GetNextPoolElement());

	if (&iPoolElement == PoolEnd)
	{
		PoolEnd = &(PoolEnd->GetPreviousPoolElement());
	}

	--Size;

	delete &iPoolElement;

} // Delete


void
PoolC::Add
(
	GenotypeA & iGenotype
)
{
	PoolElementA * Temp;
	Temp = ThePool;

	while ((Temp != 0) && (Temp->GetGenotype() > iGenotype))
	{
		Temp = &(Temp->GetNextPoolElement());
	}

	++Size;

	PoolElementA * Newrecord;
	PoolElementA * Empty = 0;

	if (ThePool == 0)
	{
		Newrecord = new PoolElementC(iGenotype, *ThePool, *PoolEnd);
		ThePool = Newrecord;
		PoolEnd = ThePool;
	}
	else
	{
		if (Temp == 0)
		{
			Newrecord = new PoolElementC(iGenotype, *PoolEnd, *Empty);
			PoolEnd = Newrecord;
		}
		else
		{
			if (&Temp->GetPreviousPoolElement() == 0)
			{
				Newrecord = new PoolElementC(iGenotype, *Empty, *Temp);
				ThePool = Newrecord;
			}
			else
			{
				Newrecord = new PoolElementC(iGenotype, Temp->GetPreviousPoolElement(), *Temp);
			}
		}
	}

} // PoolC::Insert



void PoolC::PrintToStream
(
	ostream & os
) const
{
	if (Size > 0)
	{
		PoolConstIteratorC PoolIterator(*this);

		PoolIterator.First();
		while (!PoolIterator.IsDone())
		{
			os << PoolIterator.CurrentItem() << endl;
			PoolIterator.Next();
		}
	}

} // PrintToStream


		
// Eof PoolC.cc
