/*
 * ChangeOperatorC.cc - Simple implementation of a change operator: one point mutation.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 07-Jun-1998
 *
 */

#include "Reproduction/ChangeOperatorC.h"
#include "Common/ArrayC.tpl"



ChangeOperatorC::ChangeOperatorC
(
	ConfigurationA const & iConfiguration,
	ProblemFactoryA const & iProblemFactory,
	ProblemA const & iProblem,
	RandomA const & iRandom
): ChangeOperatorA(iConfiguration, iProblemFactory, iProblem, iRandom)
{
} // ChangeOperatorC



ChangeOperatorC::~ChangeOperatorC()
{
};



void
ChangeOperatorC::operator()
(
	ArrayC<GenotypeA*> const & iArray
) const
{
	for (unsigned int i = 0; i < iArray.GetSize(); ++i)
	{
		DataA::DataIteratorA * DataIterator = iArray[i]->GetData().CreateDataIterator();

		(*DataIterator)[(Random)(iArray[i]->GetData().GetSize())] = (Random)(Configuration("Problem", "DomainSize"));
		//cerr << "  " << *(iArray.operator[](i)) << endl;

		delete DataIterator;
	}

} // ~ChangeOperatorC




// Eof ChangeOperatorC.cc
