/*
 * SelectParentsC.cc - Simple implementation of a parent selection mechanism: rank based selection.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 17-Jun-1998
 *
 */

#include "Reproduction/SelectParentsC.h"
#include "Common/ArrayC.tpl"

#define PRECISION 10000


SelectParentsC::SelectParentsC
(
	ConfigurationA const & iConfiguration,
	RandomA const & iRandom
): SelectParentsA(iConfiguration, iRandom)
{
	LinearRankingBias = Configuration("SelectParents","LinearRankingBias");

	PreWork1 = LinearRankingBias * LinearRankingBias;
	PreWork2 = 4.0 * (LinearRankingBias - 1);
	PreWork3 = 2.0 * (LinearRankingBias - 1);
	
} // SelectParentsC



SelectParentsC::~SelectParentsC()
{
};



ArrayC<GenotypeA const *>*
SelectParentsC::operator()
(
	PoolA const & iPool
) const
{
	ArrayC<GenotypeA const *>* Result = new ArrayC<GenotypeA const *>(Configuration("SelectParents", "GenotypesPerOperation"), "SelectParents");


	PoolConstIteratorA * PoolIterator = iPool.CreatePoolConstIterator();

	unsigned int i = 0;

	while (i < Configuration("SelectParents", "GenotypesPerOperation"))
	{
		double r = (double) (Random(10000) % PRECISION) / PRECISION;
		unsigned int index = (unsigned int) ((double) iPool.GetSize() * (LinearRankingBias - sqrt (PreWork1 - PreWork2 * r)) / PreWork3);
		//REM: cerr << r << " : " << index << " : " << iPool.GetSize() << endl;

		PoolIterator->First();
		for (unsigned int j = 0; j < index; ++j)
		{
			PoolIterator->Next();
		}
		(*Result)[i] = &PoolIterator->CurrentItem();

		++i;
	}

	delete PoolIterator;

	return Result;

} // operator()




// Eof SelectParentsC.cc
