/*
 * SelectSurvivorsC.cc - Simple implementation of a survivors selection mechanism: replace worst.
 *
 * Part of the Library for Evolutionary Algorithm Programming.
 *
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: J.I. van Hemert
 * Last Revision: 10-Jun-1998
 *
 */

#include "Reproduction/SelectSurvivorsC.h"
#include "Common/ArrayC.tpl"



SelectSurvivorsC::SelectSurvivorsC
(
	ConfigurationA const & iConfiguration
): SelectSurvivorsA(iConfiguration)
{

} // SelectSurvivorsC



SelectSurvivorsC::~SelectSurvivorsC()
{
};



void
SelectSurvivorsC::operator()
(
	PoolA & iPool,
	ArrayC<GenotypeA*> const & iArray
) const
{
	for (unsigned int i = 0; i < iArray.GetSize(); ++i)
	{
		iPool.Add(*(iArray[i]));
	}

	PoolIteratorA * PoolReverseIterator = iPool.CreatePoolReverseIterator();

	for (unsigned int i = 0; i < iArray.GetSize(); ++i)
	{
		PoolReverseIterator->First();
		delete &(PoolReverseIterator->CurrentItem());
		PoolReverseIterator->Delete();
	}

	delete PoolReverseIterator;

} // operator()




// Eof SelectSurvivorsC.cc
